delivered = function(playerName,cargoName,dropOffPoint)
	local playerUnit = Unit.getByName(playerName)
	local coa = HS_getUnitCoalition(playerName)
	if playerUnit then
		if mist.DBs.zonesByName[dropOffPoint] then
			distance = mist.utils.get2DDist(Unit.getPosition(playerUnit).p,mist.utils.zoneToVec3(dropOffPoint))
			radius = mist.DBs.zonesByName[dropOffPoint].radius
		else
			distance = mist.utils.get2DDist(Unit.getPosition(playerUnit).p,HS.getUnitPosition(dropOffPoint))
			radius = 50
		end
		HS.out(coa,cargoName .. ' has been delivered ' .. math.floor(distance) .. 'm from drop off point.',10)
		if distance > radius then
			HS.out('blue','You dropped the sling load too far from the drop zone! 60 penalty seconds will be added to your time.',30)
			trigger.action.outSound('Qualification Penalty.ogg')
			penalty[playerName] = 60
		end
	end
	if tasksCompleted[playerName] > 3 then
		tasksCompleted[playerName] = 5
		HS.out(coa,'Land and request to be armed by the ground crew. Then eliminate the 4 target vehicles WSW of the airfield.',10)
		trigger.action.outSound('Qualification Rearm.ogg')
		Coa = coa
		Playername = playerName
	else
		HS.out(coa,playerName .. ' has not completed the previous task.',10)
		trigger.action.outSound('Qualification Not Completed.ogg')
	end
end

takeOff = function(coa,playerName)
	HS.out(coa,playerName .. ' has taken off',10)
	if not firstTakeOff[playerName] then
		firstTakeOff[playerName] = true
		timeCounter[playerName] = timer.getTime()
		if playerName == 'Springfield 1-1' then
			HS.out(coa,'The clock for ' .. playerName .. ' is ticking.',10)
			trigger.action.outSound('Qualification Clock.ogg')
		end
	end
end

checkDistance = function(coa,playerName)
	local playerPosition = Unit.getPosition(Unit.getByName(playerName)).p
	local aiPosition = Unit.getPosition(Unit.getByName('Enfield 1-1')).p
	local distance =  mist.utils.get3DDist(playerPosition,aiPosition) 
	if distance < 100 then
		timeInFormation[playerName] = timeInFormation[playerName]+1
		HS.out(coa,playerName .. ' has been in tight formation for ' .. timeInFormation[playerName],1)
	else
		timeInFormation[playerName] = 0
	end
	if timeInFormation[playerName] < 30 then
		mist.scheduleFunction(checkDistance,{coa,playerName},timer.getTime() + 1)
	else
		HS.out(coa,'You have completed the mission in ' .. timer.getTime()+penalty[playerName] .. 's.',30)
		if timer.getTime()+penalty[playerName] < 1500 then
			missionSuccess = true
			HS.out('blue','Mission success, you passed your qualification flight.',30)
			trigger.action.outSound('Qualification Success.ogg')
		else
			HS.out('blue','You took too long to complete the flight. You need additional practice, then try the mission again.',30)
			trigger.action.outSound('Qualification Fail.ogg')
		end
	end
end

taskCompleted6 = function(coa,playerName)
	if tasksCompleted[playerName] == 5 then
		tasksCompleted[playerName] = 6
		HS.out(coa,playerName .. ' has completed the fifth task.',10)
		trigger.action.outSound('Qualification Task.ogg')
		HS.out(coa,'Your final task is to Enfield 1-1 in close formation uninterruptedly for 30s. Enfield is taking off right now.',10)
		activate = true
		HS.addReportIAS('Enfield 1-1')
		checkDistance(coa,playerName)
	else
		HS.out(coa,playerName .. ' has not completed the previous task.',10)
	end
end

hoverCompleted4 = function(coa,playerName)
	HS_stopHoverGuidance(playerName,'hover point end sideways flight')
	if tasksCompleted[playerName] == 3 then
		tasksCompleted[playerName] = 4
		sideways = false
		HS.out(coa,playerName .. ' has completed the forth task.',10)
		HS.out(coa,'Your next task is to pick up the sling load and drop it off at a location close to where your mission started. This location is marked by green smoke.',10)
		local position = mist.utils.zoneToVec3('Drop Off Zone')
		local landHeight = land.getHeight(mist.utils.makeVec2(position)) -- get altitude of ground
		position.y = landHeight + 10
		trigger.action.smoke(position,0)
		HS.addSlingLoad(playerName,'Cargo Object','Drop Off Zone',delivered)
	else
		HS.out(coa,playerName .. ' has not completed the previous task.',10)
		trigger.action.outSound('Qualification Not Completed.ogg')
	end
end

checkSideways = function(coa,playerName)
	local playerUnit = Unit.getByName(playerName)
	local heading = mist.getHeading(playerUnit)
	if heading > 110*math.pi/180 and heading < 180*math.pi/180 or heading > 290*math.pi/180 and heading < 360*math.pi/180 then
		if sideways then
			mist.scheduleFunction(checkSideways,{coa,playerName},timer.getTime() + 0.5)
		end
	else
		HS.out(coa,' You have deviated too much from sideways flight. Return to the smoke to redo the task',10)
		HS_stopHoverGuidance(playerName,'hover point end sideways flight')
		local position = mist.utils.zoneToVec3('Runway Zone')
		local landHeight = land.getHeight(mist.utils.makeVec2(position)) -- get altitude of ground
		position.y = landHeight + 10
		trigger.action.smoke(position,4)
		HS.out(coa,'Fly to the smoke mark at the western end of Kobuleti runway and hover over it for 10s with your nose oriented either along heading 145 or 325 degrees.',20)
		tasksCompleted[playerName] = 2
		HS.addHoverGuidance(playerName,{name = 'hover point above runway',position = mist.utils.zoneToVec3('Runway Zone'),minD = 5,maxD = 30,minH = 3,maxH = 10, hoverDuration = 10, code = hoverCompleted3, arguments = {coa,playerName}}) -- initialize hover instructions
	end
end

hoverCompleted3 = function(coa,playerName)
	HS_stopHoverGuidance(playerName,'hover point above runway')
	if tasksCompleted[playerName] == 2 then
		tasksCompleted[playerName] = 3
		HS.out(coa,playerName .. ' has completed the third task.',10)
		trigger.action.outSound('Qualification Task.ogg')
		sideways = true
		HS.out(coa,'Fly SIDEWAYS along the runway to the green smoke marker, and hover there for 10 s. Keep your nose oriented perpendicular to the runway.',20)
		local position = mist.utils.zoneToVec3('End Sideways Zone')
		local landHeight = land.getHeight(mist.utils.makeVec2(position)) -- get altitude of ground
		position.y = landHeight + 10
		trigger.action.smoke(position,0)
		HS.addHoverGuidance(playerName,{name = 'hover point end sideways flight',position = mist.utils.zoneToVec3('End Sideways Zone'),minD = 5,maxD = 30,minH = 3,maxH = 10, hoverDuration = 10, code = hoverCompleted4, arguments = {coa,playerName}}) -- initialize hover instructions
		mist.scheduleFunction(checkSideways,{coa,playerName},timer.getTime() + 5)
	else
		HS.out(coa,playerName .. ' has not completed the previous task.',10)
		trigger.action.outSound('Qualification Not Completed.ogg')
	end
end

hoverCompleted2 = function(coa,playerName)
	HS_stopHoverGuidance(playerName,'hover point above truck')
	if tasksCompleted[playerName] == 1 then
		tasksCompleted[playerName] = 2
		HS.out(coa,playerName .. ' has completed the second task.',10)
		trigger.action.outSound('Qualification Task.ogg')
		local position = mist.utils.zoneToVec3('Runway Zone')
		local landHeight = land.getHeight(mist.utils.makeVec2(position)) -- get altitude of ground
		position.y = landHeight + 10
		trigger.action.smoke(position,4)
		HS.out(coa,'Fly to the smoke mark at the western end of Batumi runway 13 and hover over it for 10s with your nose oriented either along heading 145 or 325 degrees.',10)
		HS.addHoverGuidance(playerName,{name = 'hover point above runway',position = mist.utils.zoneToVec3('Runway Zone'),minD = 5,maxD = 30,minH = 3,maxH = 10, hoverDuration = 10, code = hoverCompleted3, arguments = {coa,playerName}}) -- initialize hover instructions
	else
		HS.out(coa,playerName .. ' has not completed the previous task.',10)
		trigger.action.outSound('Qualification Not Completed.ogg')
	end
end

hoverCompleted1 = function(coa,playerName)
	HS_stopHoverGuidance(playerName,'hover point above Mi8')
	HS.out(coa,playerName .. ' has completed the first task. The next task is to hover above the truck while it is moving.',10)
	trigger.action.outSound('Qualification Task.ogg')
	local position = Unit.getPosition(Unit.getByName('truck Unit #2')).p
	local landHeight = land.getHeight(mist.utils.makeVec2(position)) -- get altitude of ground
	position.y = landHeight + 10
	trigger.action.smoke(position,4)
	tasksCompleted[playerName] = 1
	HS.addHoverGuidance(playerName,{name = 'hover point above truck',position = 'truck Unit #2',minD = 8,maxD = 30,minH = 3,maxH = 10, hoverDuration = 10, code = hoverCompleted2, arguments = {coa,playerName}}) -- initialize hover instructions
end

playerHasBecomeActive = function(playerName) -- this function gets executed whenever a player becomes active
	local playerUnit = Unit.getByName(playerName)
	if playerUnit then
		local coa = HS.getUnitCoalition(playerName)
		HS.out(coa,playerName .. ' has become active.',10)
		HS.addCheckForIncoming(playerName) -- make sure that copilot gives warning when an enemy missile is launched
		HS.addCheckForSinkRate(playerName) -- make sure that copilot gives warning when dangerously close to VRS
		HS.onTakeOffDo(playerName,takeOff,{coa,playerName}) -- give message when unit takes off
		HS.onTouchDownDo(playerName,HS.out,{coa,playerName .. ' has landed',10}) -- give message when unit touches down
		firstTakeOff[playerName] = false
		timeCounter[playerName] = nil
		tasksCompleted[playerName] = 0
		penalty[playerName] = 0
		timeInFormation[playerName] = 0
		HS_shipsToLandOn["Ship Unit #1"] = {playerName = playerName, code = playerHasLandedOnShip,arguments = {coa,playerName}}
		HS.addHoverGuidance(playerName,{name = 'hover point above Mi8',position = mist.utils.zoneToVec3('hover zone above Mi8'),minD = 5,maxD = 30,minH = 5,maxH = 10, hoverDuration = 10, code = hoverCompleted1, arguments = {coa,playerName}}) -- initialize hover instructions
	end
end

playerHasBecomeInactive = function(playerName) -- this function gets executed whenever a player becomes inactive
	local coa = HS.getUnitCoalition(playerName)
	HS.out(coa,playerName .. ' has become inactive.',10)
end

smokeMarker1Blue = function()
	local position = mist.utils.zoneToVec3('hover zone above Mi8')
	local landHeight = land.getHeight(mist.utils.makeVec2(position)) -- get altitude of ground
	position.y = landHeight + 15
	trigger.action.smoke(position,4)
end

-- end of functions

-- define sound files

HS.setIncomingAudio('incoming.wav')
HS.setRadioStaticAudio('RadioClickAfter.wav')
HS.setBeaconAudio('VHF_WarningTone.wav')
HS.setSinkRateAudio('sinkRate.wav')
HS.setHoverDirectionsAudio('up.wav','down.wav','left.wav','right.wav','forward.wav','back.wav')
HS.setCargoHookedAudio('Hooked.wav')

-- end define sound files

-- set up troops and structures at airfields

firstTakeOff = {}
timeCounter = {}
tasksCompleted = {}
penalty = {}
timeInFormation = {}
activate = false
missionSuccess = false

local count

for count = 1, #mist.DBs.MEunitsByCat.helicopter do
	HS.onBecomesActiveDo(mist.DBs.MEunitsByCat.helicopter[count].unitName,playerHasBecomeActive,{mist.DBs.MEunitsByCat.helicopter[count].unitName})
	HS.onBecomesInactiveDo(mist.DBs.MEunitsByCat.helicopter[count].unitName,playerHasBecomeInactive,{mist.DBs.MEunitsByCat.helicopter[count].unitName})
end
